//+------------------------------------------------------------------+
//|                                               ED_Kaiser_LPF.mq4  |
//|                   Copyright  2009, Eugene Dvoskin as begemot61  |
//|                                 Kaiser Window FIR Digital Filter |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, Eugene Dvoskin"
#property link      ""
// ED_Kaiser_LPF
#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Magenta

//-------------------------input parameters------------------------------------

extern double    PassBandBars=250.0;// filter pass band in BARS
extern double    StopBandBars=2.6;// filter stop band in BARS
extern double    StopBandAttenuation=21.0; // stop band attenuation in dB
extern int       Price_applied=0; //0-Close price, 1-Open price, 2-High price, 3-Low price, 4-Median, 5-Typical, 6-Weighted

//PassBand=1/PassBandBars -> relative range of filter pass band, possible range is from 0.0 to (0.5-StopBand)
//                           It is normalized and it's maximum value (0.5-StopBand) is 1/2                     
//StopBand=1/StopBandBars -> relative range of filter stop band, possible range is from 0.0 to (0.5-PassBand)
//                           It is normalized and it's maximum value (0.5-PassBand) is 1/2                 
// (PassBand + StopBand) < 0.5, Relative F_sampling=1, Abs F_sampling= Bar's frequency
// If (PassBand + StopBand) >= 0.5, they would be normalized

//	  Gain(dB)
//		|	     ..    <------pass band ripple (in dB)
//	 0dB|....  .  .                                     PassBand=Fpass;
//		|    ..    .                                    StopBand=Fstop-Fpass; width of stop band 
//		|	       	 .                                  Fstop<0.5*F_sample
//		|              .         
//		|               .     ..       <-----stop band attenuation (in dB) 
//		|                .  .    .
//	    |---------+-------+--------*-------+----------------------------------+------Frequency
//		0       Fpass   Fstop          0.5F_sample                          F_sample 
//		
			   
//--------------------------------------------------------------------
double  alfa; //---Kaiser window parameter
double  PassBand ;
double  StopBand ;
double  NormCoef[];//---- normalized filter coefficients
double  PassBandRipple=30.0; // pass band ripple--> +/- PassBandRipple (in dB)
double  pi=3.1415926535;
double  Response=0.0; //last calculated value of filter response
int     FilterLength;

//---- buffers
double FilterBuffer1[];
   
   
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,FilterBuffer1);
//---- 
   Normalization(); //  normalized filter coefficients for the 1st time
//----
   SetIndexDrawBegin(0,FilterLength+10);
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int   limit, i;
   int   counted_bars=IndicatorCounted(); //amount of bars shanged
   if(Bars<=(FilterLength+1)) return(0);  //not enough bars for calculations
   if(counted_bars < 0) return (0);       //eror protection
   if(counted_bars > 0) counted_bars--;
   limit=Bars-counted_bars-1;
      for (i = limit;i>=0;i--)                  // Cycle for uncalculated bars
     {
      FilterBuffer1[i] = FilterResponse(i);     // Value of 0 buffer on i-th bar
     }
   return(0);
  }

//+------------------------------------------------------------------+
//| Custom functions prototypes                                      |
//+------------------------------------------------------------------+

void FilterOrder() // filter length estimations
	{
	 double sbripple, pbripple, ripple, attenuation,bandwidth, d, n, x;
	 
	 PassBand=1/PassBandBars;
	 StopBand=1/StopBandBars;
    // to check that filter parameters are entered properly
    // if not --> correct them to the closest values
	 bandwidth=PassBand+StopBand;
    if (bandwidth >=0.5) 
       {
        PassBand=0.5*PassBand/bandwidth;
        StopBand=0.5*StopBand/bandwidth;
       }
	 sbripple=MathPow(10.0, (-0.05*StopBandAttenuation));
	 pbripple=MathPow(10.0, (0.05*PassBandRipple))-1.0;
	 ripple=MathMin(sbripple, pbripple); //whatever is smaller is our worst case
	 attenuation=-20*MathLog(ripple)/MathLog(10); //this would be our real stop band attenuation
	 if(NormalizeDouble(attenuation,5)<=21.0) {alfa=0.0; d=0.9222;}
	 if(NormalizeDouble(attenuation,5)>50.0) 
		{
		 alfa=0.1102*(attenuation-8.7); d=(attenuation-7.95)/14.36;
		}
	 if(NormalizeDouble(attenuation,5)>21.0 && NormalizeDouble(attenuation,5)<=50) 
		{
	    alfa=(0.5842*MathPow((attenuation-21.0), 0.4))+(0.07886*(attenuation-21.0));
	    d=(attenuation-7.95)/14.36;
		}
	 n=(d/StopBand)+1.0;
	 x=MathRound(n);
	 if(MathMod(x,2.0)<1) FilterLength=x;
	 else FilterLength=x-1;
	 
	}
//+---------------------------------------------------------------------
double Io(double x)      // modified zero order Bessel function (hyperbolic)  
   {			         // it is Io(x), not Jo(x)  !!!!!!!!!
	double besselAccuracy=0.000001; 
	double bessel=1.0;
	double summ, temp;
	double k=2.0; 
	double factorial=1.0;  
	temp=x/2;
	summ=temp*temp;
	bessel+=summ;
	while (summ>besselAccuracy) //will do until we have small error
		{
		factorial=factorial*k;
		temp*=x/2;
		summ=temp/factorial;
		summ=summ*summ;
		bessel+=summ;
		k+=1.0;
		}
	return (bessel);   
   }
//+---------------------------------------------------------------------   
void Normalization()
  {
   double norm, filter, Ioalfa, temp;
   double filterCoeff[], kaiserWindow[];
   int i, M;

   FilterOrder();
   M=FilterLength/2 ; // half of filter length
   
  // ---------------Window function----------------------
   ArrayResize(kaiserWindow,M+1); 
   ArrayInitialize(kaiserWindow,0.0);
   Ioalfa=Io(alfa); norm=M ;
   for(i=1; i<=M; i++)
      {
       temp=i/norm;
       kaiserWindow[i]=Io(alfa*MathSqrt(1-(temp*temp)))/Ioalfa ;
      } 
   
   // --------------filter coefficients-------------------
   ArrayResize(filterCoeff, FilterLength+1);
   ArrayInitialize(filterCoeff,0.0);
   filterCoeff[0]=2.0*(PassBand+(0.5*StopBand));
   norm=filterCoeff[0];
   temp=pi*filterCoeff[0];
   for(i=1; i<=M; i++) 
      {
       filterCoeff[i]=MathSin(i*temp)*kaiserWindow[i]/(i*pi);
       norm =norm + (2*filterCoeff[i]); // to be used in normalization
      }
   
   // --------------casual conversion and normalization---------------------
   ArrayResize(NormCoef, FilterLength+1);
   ArrayInitialize(NormCoef,0.0);
   for(i=M+1; i<=FilterLength; i++) filterCoeff[i] = filterCoeff[i - M]  ;
	for(i=0; i<=M-1; i++) filterCoeff[i] = filterCoeff[FilterLength - i] ;
	filterCoeff[M] = 2.0*(PassBand+(0.5*StopBand)) ;
	for(i=0; i<=FilterLength; i++) NormCoef[i] = filterCoeff[i]/norm; 
   
  } // End of Normalization()
  
 //+---------------PriceSelect-----------------------------------------------------
double PriceSelect(int index_of_bar, int Price_applied)
   {
    double priceValue;
    
      switch(Price_applied)// to select desired price for filter calculations
      { 
       case 0: priceValue=Close[index_of_bar]; break;
       case 1: priceValue=Open[index_of_bar]; break;
       case 2: priceValue=High[index_of_bar]; break;
       case 3: priceValue=Low[index_of_bar];  break; 
       case 4: priceValue=(Low[index_of_bar]+High[index_of_bar])/2;  break; //Median
       case 5: priceValue=(Low[index_of_bar]+High[index_of_bar]+Close[index_of_bar])/3;  break; //5-Typical
       case 6: priceValue=(Open[index_of_bar]+Low[index_of_bar]+High[index_of_bar]+Close[index_of_bar])/4;  break; // Weighted
       default: priceValue=Close[index_of_bar];
      }
    return(priceValue);
   }  // End of PriceSelect()
   
 //+---------------filter response value for Bars=index_of_bar--------------------- 
 
double FilterResponse(int index_of_bar)  
   {
   double valueBuf, temp, temp1;
   double Response=0.0;
   int i, filterlength=0;

	 while (filterlength<=FilterLength)
      {
         valueBuf=PriceSelect((index_of_bar+filterlength), Price_applied);
         Response=Response+valueBuf*NormCoef[filterlength]; 
         filterlength++; 
      } 
 
    return(Response);    
   } // End of FilterResponse



